#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Mon Feb 12 08:42:03 2018

@author: lenaboysen

Modified t-test after Zwiers and von Storch 1995, J.Clim 8, p.336-351
Based on Lorenz et al., GRL, 2016: 
R Source code: https://github.com/ruthlorenz/stat_tests_correlated_climdata


## =========================================================
USAGE:
 ## =========================================================
 
x, y dimensions: lon, lat, time    
    
test3d = modTtest3d(x,y,alternative="greater",conf_level=0.95) 

with
alternatives: less, greater, two_sided
conf_level: 0.90, 0.95, 0.99... 

Defaults: 
mu: 0 
paired: False 
var_equal: True
time_vals: None 

"""

#%% Import Packages
import numpy as np
import sys as sys
from scipy.stats import t
#import xarray as xr

#%%
'''
==========================================================
FUNCTIONS
==========================================================
'''

#%% modified t-test after Zwiers and von Storch 1995, J.Clim 8, p.336-351conf_level
### Translated R-code from Lorenz et al., 2016, GRL.: Source code from
### https://github.com/ruthlorenz/stat_tests_correlated_climdata


#x = ds_x
#y = None
#alternative = 'greater'

def modTtest3d(x, y = None, alternative = 'two_sided',
               mu = 0, paired = False, var_equal = True, conf_level = 0.95,
               time_vals = None): 
    
    alternatives = ['two_sided', 'less', 'greater']
    if alternative not in alternative:
        raise ValueError("Invalid alternatvie type. Expected one of: %s" % alternatives)


    if mu is not None and np.size(mu) != 1 or np.isnan(mu):
        sys.exit('mu  must be a single number')
    
    if conf_level is not None and np.size(conf_level) != 1 or not np.isfinite(conf_level) or conf_level < 0 or conf_level > 1:
        sys.exit('conf_level must be a single number between 0 and 1')


    #find grid point which is not missing
    rowcol = np.argwhere(np.isfinite(x)) # indices need to be returned!
    lonind = rowcol[1,0]
    latind = rowcol[1,1]
    
    
    if y is not None:
        #print 'true'
        dname = get_var_name(x=x) + ' and ' + get_var_name(y=y)
        
        if(paired == True):
            xok = np.array_equal(np.isfinite((x[lonind,latind,:]),np.isfinite(y[lonind,latind,:]))) ###  i.e., have no missing values 
            yok = xok
        else :
            yok = np.isfinite(y[lonind,latind,:]) #!is.na(y[lonind,latind,])
            xok = np.isfinite(x[lonind,latind,:]) 

        y = y[:,:,yok]
    else:
        dname = str(x)
        if paired == True:
            sys.exit('Error: y is missing for paired test')
        xok = np.isfinite(x[lonind,latind,:])
        yok = None    
    
    x = x[:,:,xok]
    
    
    if paired == True:
        x = x - y
        y = None
	
    dimsX = x.shape
    nx = dimsX[2]
    
    if time_vals is not None:
        if nx != time_vals:
            sys.exit('Error: wrong dimension, the time in X does not correspond to time_vals')
    
    if y is not None: 
        dimsY = y.shape
        ny = dimsY[2]
        
        if dimsX[1] != dimsY[1]:
            sys.exit('Error: X and Y have different number of latitudes')
        elif dimsX[0] != dimsY[0]:
            sys.exit('Error: X and Y have different number of longitudes')

    nlat = dimsX[1]
    nlon = dimsX[0]
    
    mx = np.empty((nlon,nlat,1))
    vx = np.empty((nlon,nlat,1))
    
    mx[:,:,0] = np.nanmean(x,axis=2) # mean over time, lon lat remain
    vx[:,:,0] = np.nanvar(x,axis=2)
    
    
    if y is None: 
        if nx < 2:
            sys.exit('not enough x observations')
        #------ Lag-1 Autocorrelation ------#
        r1 = calc_lag1Corr(x=x,xAve=mx)
        if nx >= 30:
            #------ Equivalent Sample Size ------#
            ne = calc_EqvSamplSize_large(x=x,r1=r1)
            df = ne - 1
            stderr = np.sqrt(vx / ne)
            if np.all(stderr < 10*np.finfo(float).eps*np.abs(mx)):
                sys.exit('data are essentially constant')
            else:
                tstat = (mx - mu) / stderr
                method = 'One Sample modified t-test for spatial data'
                estimate =mx
                
        else:
             #------ Lookup table test, df undefined ------#
            df = np.empty(1)
            stderr = np.sqrt(vx / nx)
            tstat = (mx - mu)/stderr
            tcrit = lookupTcrit(r1,1-conf_level,nx)
            if paired == True:
                method = 'Paired modified t-test for spatial data'
                estimate = mx
            else:
                method = 'One Sample modified lookup table t-test for spatial data'
                estimate = mx
    else: # y is not None
         	#------ Two-sample test ------#
        dimsY = y.shape
        ny = dimsY[2]
         
        my = np.empty((nlon,nlat,1))
        vy = np.empty((nlon,nlat,1))
        my[:,:,0] = np.nanmean(y,axis=2)
        vy[:,:,0] = np.nanvar(y,axis=2)
         
        if nx < 2:
            sys.exit('not enough x observations')
        if ny < 2:
            sys.exit('not enough y observations')
        if var_equal is None or var_equal is False:
            sys.exit('the two sample modified t-test assumes equal variance')
        if nx + ny < 3 :
            sys.exit('not enough observations')
        #------ Pooled sample variance, Eq. 13 ------#   
        sp2 = calc_poolSampVar(x=x,xAve=mx,y=y,yAve=my)
        estimate = my - mx
        #names(estimate) <- "Difference between x and y"
    
        if nx + ny >= 30:
            #------ Lag-1 Autocorrelation ------#
            r1 = calc_lag1Corr(x=x,xAve=mx,y=y,yAve=my)
            #------ Equivalent Sample Size ------#
            EqvSize = calc_EqvSamplSize_large(x=x,r1=r1,y=y) ## default: x,r1,y=None
            ne = EqvSize['NE']
            xne_p = EqvSize['Xne_p']
            yne_p = EqvSize['Yne_p']
            df = xne_p+yne_p-2
            stderr = (np.sqrt(sp2)*ne)
            tstat = (my-mx)/stderr	#Eq. 12
            method = 'Two Sample modified t-test for spatial data'
        else: 
            #------ Lookup table test, df undefined ------#
            df = np.empty(1)
            stderr = (np.sqrt(sp2)*(np.sqrt(1/float(nx)+1/float(ny))))
            tstat = (my-mx)/stderr		#Eq.16
            r1 = calc_lag1Corr(x=x,xAve=mx,y=y,yAve=my)
            tcrit = lookupTcrit(r1,1-conf_level,nx+ny)
            method = "Two Sample modified lookup table t-test for spatial data"
       
        if np.all(stderr < 10*np.finfo(float).eps*np.abs(mx)):
            sys.exit('data are essentially constant')
     
              
    cint = np.empty((2,nlon,nlat,1))
    if np.all(np.isnan(df)):
        #------ Lookup table test, pval undefined ------#
        pval = np.empty(1)
        if alternative == 'less':
            cint[0,:,:,:] = -np.inf
            cint[1,:,:,:] = tcrit
        elif alternative == 'greater':
            cint[0,:,:,:] = tcrit
            cint[1,:,:,:] = np.inf
        else:
            ci = tcrit
            cint[0,:,:,:] = tstat - ci
            cint[1,:,:,:] = tstat + ci
    else:
        if alternative == 'less': # only left tail
            pval = 1-t.cdf(np.abs(tstat), df) # probability function = CDF // pt() in R
            cint[0,:,:,:] = -np.inf
            cint[1,:,:,:] = tstat + t.ppf(conf_level, df = df) # qunatile function = inverse of CDF // qt in R
        elif alternative == 'greater': # only right tail
            pval = t.cdf(np.abs(tstat), df) 
            cint[0,:,:,:] = tstat - t.ppf(conf_level, df = df)
            cint[1,:,:,:] = np.inf
        else:
            pval = (t.sf(np.abs(tstat), df)*2)
            alpha = 1 - conf_level
            ci = t.ppf(1 - alpha/2, df = df)
            cint[0,:,:,:] = tstat - ci
            cint[1,:,:,:] = tstat + ci
            
     
    cint[0,:,:,:] = mu + cint[0,:,:,:] * stderr 
    cint[1,:,:,:] = mu + cint[1,:,:,:] * stderr
    
#    print 'tstat='
#    print tstat
#    print 'df='
#    print df
#    print 'pval'
#    print pval
#    print cint
#    print estimate
#    print alternative
#    print method
#    print dname
#    print mu

    ## Write out data in dictionary
    rval = {}
    if nx + ny >= 30:
        rval['t-statistic'] = tstat[:,:,0]   # t-value
        rval['parameter_df'] = df[:,:,0]     # sample size? 
        rval['pvalue'] = pval[:,:,0]         # Probability
        rval['conf_interval'] = cint[:,:,:,0]  # qt() of where 0.95, 0.99 ... 
        rval['estimate'] = estimate[:,:,0]   
        rval['alternative'] = alternative
        rval['method'] = method
        rval['data_name'] = dname      
        if paired == True or y is not None:
            name = 'difference_in_means_mu'
        else:
            name = 'mean_mu'
        rval[name] = mu
    else:
        rval['t-statistic'] = tstat[:,:,0]   # t-value
        rval['parameter_df'] = df[0]     # sample size? 
        rval['pvalue'] = pval[:,:,0]         # Probability
        rval['conf_interval'] = cint[:,:,:,0]  # qt() of where 0.95, 0.99 ... 
        rval['estimate'] = estimate[:,:,0]   
        rval['alternative'] = alternative
        rval['method'] = method
        rval['data_name'] = dname      
        if paired == True or y is not None:
            name = 'difference_in_means_mu'
        else:
            name = 'mean_mu'
        rval[name] = mu        
    return(rval)    


        
#%% CALC lag1Corr 
# Calculate lag 1 correlation coefficient
# for modified t-test (large and small samples)
# Zwiers and von Storch, 1995, J.Clim, Eq. 14 (two sample case)

def calc_lag1Corr(x, xAve, y = None,yAve = None):
    
    dimsX = x.shape
    xnyrs = dimsX[2]
    
    ## Check if second data set has same #lat
    if y is not None:
        dimsY = y.shape
        ynyrs = dimsY[2]
    
        if dimsX[1] != dimsY[1]:
            sys.exit('Error: X and Y have different number of latitudes')
        elif dimsX[0] != dimsY[0]:  
            sys.exit('Error: X and Y have different number of longitudes')
        
    nlat = dimsX[1]
    nlon = dimsX[0]
    
    xsumsq = np.zeros((nlon,nlat,1))
    xsumlag = np.zeros((nlon,nlat,1))    

    if y is not None:
        ysumsq = np.zeros((nlon,nlat,1))
        ysumlag = np.zeros((nlon,nlat,1))


    r1 = np.empty((nlon,nlat,1))

    #Sum of Squares
    for yr in range(0,xnyrs):
        xsumsq = xsumsq + (x[:,:,[yr]] - xAve)**2
        
        
    if y is not None:
        for yr in range(0,ynyrs):
            ysumsq = ysumsq + (y[:,:,[yr]] - yAve)**2
            
        xysumsq = xsumsq+ysumsq
        
    for yr in range(1,xnyrs):
        xsumlag = xsumlag + ((x[:,:,[yr]]-xAve)*(x[:,:,[(yr-1)]]-xAve))
	   
    if y is not None:
        for yr in range(1,ynyrs):
            ysumlag = ysumlag+ ((y[:,:,[yr]]-yAve)*(y[:,:,[(yr-1)]]-yAve))
		
        xysumlag = xsumlag + ysumlag
        r1 = xysumlag/xysumsq
        
    elif y is None:
	    r1 = xsumlag/xsumsq
        
    return r1


#%%
### 
# Calculate equivalent sample size 
# for Modified t-test for large samples (ne>=30)
# Zwiers and von Storch, 1995, J.Clim, Eq. 10

def calc_EqvSamplSize_large(x,r1,y=None):
    
    dimsX = x.shape
    xnyrs = dimsX[2]

    if y is not None:
        dimsY = y.shape
        ynyrs = dimsY[2]
    
        if dimsX[1] != dimsY[1]:
            sys.exit('Error: X and Y have different number of latitudes')
        elif dimsX[0] != dimsY[0]:  
            sys.exit('Error: X and Y have different number of longitudes')

    nlat = dimsX[1]
    nlon = dimsX[0]

    
    xne_p = np.empty((nlon,nlat,1))
    ne    = np.empty((nlon,nlat,1))
        
    if xnyrs > 50 and np.any(np.abs(r1) != 1.0): ## one sample case 
        xne = xnyrs * ((1 - r1) / (1 + r1))        
    else: # two sample case
        den = np.zeros((nlon,nlat,1))
        for n in range(1,xnyrs-1):
            if np.all(np.isfinite(x[:,:,n])): 
                den =den + (1 - float(n)/xnyrs)*r1**n  #
                
        xne = xnyrs / ( 1 + 2*den )       

    if y is not None:
        yne_p = np.zeros((nlon,nlat,1))
        if ynyrs > 50 and np.any(np.abs(r1) != 1.0):
            yne = ynyrs * ((1 - r1) / (1 + r1))
        else:
            deny = np.zeros((nlon,nlat,1))
            for n in range(1, ynyrs-1):
                if np.all(np.isfinite(y[:,:,n])) : # if not all np.nan
                    deny = deny + (1 - float(n)/ynyrs )* r1**float(n)
    
            yne = ynyrs / ( 1 + 2*deny )
    
    
    for lat in range(0,nlat):
        for lon in range(0,nlon):
            if np.all(np.isnan(x[lon,lat,:])):
                xne_p[lon,lat,0] = np.nan
                if y is not None:
                    yne_p[lon,lat,0] = np.nan
                ne[lon,lat,0] = np.nan    
            else: 
                if r1[lon,lat,0] == 0:
                    xne_p[lon,lat,0] = xnyrs
                    if y is not None:
                        yne_p[lon,lat,0] = ynyrs
                else: # r1 not 0
                    if xne[lon,lat,0] <= 2:
                        xne_p[lon,lat,0] = 2
                    elif (xne[lon,lat,0]>2) and (xne[lon,lat,0]<=xnyrs):
                        xne_p[lon,lat,0] = xne[lon,lat,0]
                    else:
                        xne_p[lon,lat,0] = xnyrs
                        
                if y is not None:
                    if yne[lon,lat,0] <= 2:
                        yne_p[lon,lat,0] == 2
                    elif (yne[lon,lat,0]>2) and (yne[lon,lat,0]<=ynyrs):
                        yne_p[lon,lat,0] = yne[lon,lat,0]
                    else:
                        yne_p[lon,lat,0] = ynyrs
                        
                  
                    
    if y is not None:
        ne = np.sqrt(1/xne_p+1/yne_p)
        
        Eqsamplesize = {}
        Eqsamplesize['Xne_p'] = xne_p
        Eqsamplesize['Yne_p'] = yne_p
        Eqsamplesize['NE'] = ne
        return Eqsamplesize
    else:
        ne = xne_p
        return ne           
    
    
#%%
# function for modified t-test for small samples according to
# Zwiers and von Storch, 1995, J. Clim, Tables 6-10
# to return critical t-values from lookup tables

def lookupTabs(nobs):
    stat_val = np.empty((27,5))
    
    #----------------------------------------------------------------------
    if nobs <= 12 : 
        print 'Data not available in lookup table for n= ' + str(nobs)
    #----------------------------------------------------------------------  		    
    elif (nobs > 12 ) and ( nobs <= 17 ):
    #----------------------------------------------------------------------
        # 80% significance for n=15
        stat_val[:,0] = (np.nan, 1.87, 1.88, 1.92, 1.99,
               2.04, 2.09, 2.18, 2.26, 2.35, 2.48, 2.59, 2.75,
               2.96, 3.20, 3.46, 3.66, 3.92, 4.23, 4.59, 4.83,
               5.17, 5.37, np.nan, np.nan, np.nan, np.nan)
        # 90% significance for n=15
        stat_val[:,1] = (np.nan, 2.57, 2.59, 2.66, 2.76,
               2.89, 2.98, 3.15, 3.30, 3.47, 3.71, 3.96, 4.25,
               4.60, 5.10, 5.56, 5.60, 6.54, 6.93, 7.45, 7.91,
               8.39, 8.57, np.nan, np.nan, np.nan, np.nan)
        # 95% significance for n=15
        stat_val[:,2] = (np.nan, 3.44, 3.47, 3.61, 3.75,
               3.88, 4.05, 4.44, 4.68, 5.16, 5.55, 6.00, 6.50,
               7.17, 7.93, 8.74, 9.36, 9.84, 10.6, 11.2, 12.1,
               12.5, 12.3, np.nan, np.nan, np.nan, np.nan)
        # 98% significance for n=15
        stat_val[:,3] = (np.nan, 5.02, 5.12, 5.43, 5.67,
               5.87, 6.49, 7.35, 7.97, 8.92, 10.1, 10.5, 11.1,
               12.8, 13.7, 14.9, 16.3, 17.4, 17.9, 18.6, 19.8,
               19.3, 18.8, np.nan, np.nan, np.nan, np.nan)
        # 99% significance for n=15
        stat_val[:,4] = (np.nan, 6.82, 7.01, 7.42, 7.70,
               8.41, 9.41, 10.6, 11.6, 13.7, 14.9, 15.5, 16.6,
               18.4, 19.8, 20.9, 23.0, 23.7, 25.0, 25.9, 26.8,
               26.3, 24.3, np.nan, np.nan, np.nan, np.nan)
    #----------------------------------------------------------------------
    elif  nobs > 17 and nobs <= 22 :
    #----------------------------------------------------------------------
        # 80% significance for n=20
        stat_val[:,0] = (np.nan, np.nan, 1.70, 1.71, 1.75,
               1.80, 1.84, 1.88, 1.97, 2.08, 2.18, 2.27, 2.42,
               2.57, 2.79, 3.02, 3.38, 3.71, 4.13, 4.47, 4.85,
               5.37, 5.71, 5.99, np.nan, np.nan, np.nan)
        # 90% significance for n=20
        stat_val[:,1] = (np.nan, np.nan, 2.26, 2.28, 2.35,
               2.44, 2.52, 2.59, 2.76, 2.88, 3.06, 3.26, 3.63,
               3.82, 4.20, 4.74, 5.28, 5.93, 6.64, 7.36, 8.03,
               8.70, 9.13, 9.65, 9.67, np.nan, np.nan)
        # 95% significance for n=20
        stat_val[:,2] = (np.nan, np.nan, 2.84, 2.86, 2.98,
               3.11, 3.19, 3.34, 3.59, 3.82, 4.15, 4.61, 5.20,
               5.57, 6.25, 7.19, 8.25, 9.10, 10.2, 11.4, 12.1,
               13.2, 13.8, 14.2, np.nan, np.nan, np.nan)
        # 98% significance for n=20
        stat_val[:,3] = (np.nan, np.nan, 3.80, 3.89, 4.03,
               4.21, 4.38, 4.71, 5.30, 5.72, 6.51, 7.18, 8.59,
               8.94, 10.5, 12.8, 14.9, 16.3, 17.3, 18.7, 19.2,
               21.3, 21.4, 20.7, np.nan, np.nan, np.nan) 
        # 99% significance for n=20
        stat_val[:,4] = (np.nan, np.nan, 4.70, 4.83, 5.12,
               5.26, 5.45, 5.91, 7.22, 8.28, 9.44, 11.0, 12.1,
               12.7, 15.9, 19.7, 23.0, 24.9, 25.7, 26.3, 26.3,
               27.8, 27.3, 26.8, np.nan, np.nan, np.nan) 

    #----------------------------------------------------------------------
    elif nobs > 22 and nobs <= 27 :
            #----------------------------------------------------------------------

        # 80% significance for n=25
        stat_val[:,0] = (np.nan, np.nan, np.nan, 1.62,
               1.64, 1.67, 1.68, 1.72, 1.79, 1.87, 1.96, 2.08,
               2.22, 2.36, 2.56, 2.74, 3.05, 3.41, 3.77, 4.26,
               4.72, 5.28, 5.82, 6.49, 6.66, np.nan, np.nan)
        # 90% significance for n=25
        stat_val[:,1] = (np.nan, np.nan, np.nan, 2.15,
               2.17, 2.22, 2.24, 2.31, 2.40, 2.56, 2.69, 2.85,
               3.07, 3.32, 3.72, 4.07, 4.59, 5.23, 5.96, 6.96,
               7.67, 8.72, 9.44, 10.3, 10.4, np.nan, np.nan)
        # 95% significance for n=25
        stat_val[:,2] = (np.nan, np.nan, np.nan, 2.65,
               2.70, 2.76, 2.81, 2.90, 3.05, 3.29, 3.55, 3.74,
               4.13, 4.52, 5.23, 5.88, 6.72, 8.00, 9.21, 11.0,
               11.9, 13.5, 14.2, 15.2, 15.1, np.nan, np.nan)
        # 98% significance for n=25
        stat_val[:,3] = (np.nan, np.nan, np.nan, 3.34,
               3.42, 3.52, 3.67, 3.91, 4.07, 4.49, 4.78, 5.28,
               6.16, 6.88, 8.26, 9.54, 11.4, 13.8, 15.6, 18.9,
               20.6, 22.0, 22.1, 22.9, 22.7, np.nan, np.nan) 
        # 99% significance for n=25
        stat_val[:,4] = (np.nan, np.nan, np.nan, 3.87,
               3.98, 4.16, 4.47, 4.77, 5.11, 5.80, 6.20, 7.08,
               8.80, 10.1, 12.0, 14.1, 17.6, 19.6, 22.6, 26.1,
               28.6, 30.8, 29.9, 29.8, 29.3, np.nan, np.nan) 

    #----------------------------------------------------------------------
    elif nobs > 27 and nobs <= 37 : 
    #----------------------------------------------------------------------

        # 80% significance for n=30
        stat_val[:,0] = (np.nan, np.nan, np.nan, 1.54,
               1.55, 1.59, 1.64, 1.66, 1.71, 1.76, 1.82,
               1.91, 2.05, 2.22, 2.38, 2.61, 2.79, 3.24, 3.55, 3.94, 
               4.49, 5.25, 5.85, 6.49, 7.26, 7.31, np.nan)
        # 90% significance for n=30
        stat_val[:,1] = (np.nan, np.nan, np.nan, 2.03,
               2.04, 2.09, 2.13, 2.18, 2.26, 2.38, 2.43,      
               2.58, 2.79, 3.07, 3.35, 3.67, 4.09, 4.80, 5.31, 6.14,  
               7.15, 8.50, 9.22, 10.5, 11.4, 11.4, np.nan)
        # 95% significance for n=30
        stat_val[:,2] = (np.nan, np.nan, np.nan, 2.50,
               2.50, 2.56, 2.62, 2.71, 2.82, 2.99, 3.12, 
               3.34, 3.52, 4.05, 4.48, 5.05, 5.92, 7.01, 7.79, 9.22, 
               11.0, 13.0, 14.2, 15.6, 16.5, 16.5, np.nan)
        # 98% significance for n=30
        stat_val[:,3] = (np.nan, np.nan, np.nan, 3.07,
               3.10, 3.19, 3.30, 3.44, 3.60, 3.95, 4.26,
               4.40, 4.81, 5.68, 6.58, 7.85, 8.92, 11.4, 12.6, 14.8,  
               18.9, 22.2, 23.0, 24.6, 25.1, 25.0, np.nan)
        # 99% significance for n=30
        stat_val[:,4] = (np.nan, np.nan, np.nan, 3.58,
               3.58, 3.69, 3.80, 4.01, 4.21, 4.77, 5.27, 5.37,
               6.17, 7.51, 8.80, 11.3, 13.6, 16.4, 19.2, 22.0, 27.6, 
               30.4, 29.7, 31.3, 30.9, 30.8, np.nan)


    #----------------------------------------------------------------------
    elif nobs > 37 and nobs <= 52 :
    #----------------------------------------------------------------------

        # 80% significance for n=45
        stat_val[:,0] = (np.nan, np.nan, np.nan, np.nan,
               1.45, 1.47, 1.50, 1.53, 1.60, 1.64, 1.70,
               1.77, 1.87, 1.96, 2.12, 2.31, 2.55, 2.73, 3.03, 3.45,  
               3.89, 4.49, 5.35, 6.42, 7.82, 8.77, np.nan)
        # 90% significance for n=45
        stat_val[:,1] = (np.nan, np.nan, np.nan, np.nan,
               1.89, 1.92, 1.96, 2.02, 2.09, 2.15, 2.24,
               2.33, 2.48, 2.66, 2.85, 3.11, 3.53, 3.79, 4.39, 5.05, 
               5.80, 6.87, 8.43, 10.4, 12.8, 13.8, np.nan)
        # 95% significance for n=45
        stat_val[:,2] = (np.nan, np.nan, np.nan, np.nan,
               2.34, 2.34, 2.40, 2.46, 2.53, 2.65, 2.77,
               2.89, 3.07, 3.32, 3.60, 3.96, 4.50, 4.99, 5.82, 6.98,
               8.33, 10.1, 13.1, 15.2, 18.8, 20.0, np.nan)
        # 98% significance for n=45
        stat_val[:,3] = (np.nan, np.nan, np.nan, np.nan,
               2.79, 2.82, 2.97, 3.04, 3.08, 3.28, 3.45,
               3.60, 3.81, 4.26, 4.70, 5.26, 6.32, 6.99, 8.73, 10.6,
               14.2, 17.6, 21.9, 24.6, 28.6, 27.9, np.nan)
        # 99% significance for n=45
        stat_val[:,4] = (np.nan, np.nan, np.nan, np.nan,
               3.20, 3.23, 3.34, 3.48, 3.56, 3.66, 3.96,
               4.25, 4.43, 5.25, 5.71, 6.32, 7.65, 9.56, 12.0, 14.5,
               20.6, 26.5, 29.8, 33.5, 36.3, 33.8, np.nan)

    #----------------------------------------------------------------------
    elif nobs > 52 and nobs <= 67 :
    #----------------------------------------------------------------------
            
        # 80% significance for n=60
        stat_val[:,0] = (np.nan, np.nan, np.nan, np.nan,
               np.nan, 1.44, 1.47, 1.49, 1.55, 1.58, 1.66,
               1.70, 1.78, 1.91, 2.02, 2.22, 2.36, 2.57, 2.86, 3.14,
               3.60, 4.01, 4.91, 6.00, 7.33, 9.47, 9.93)
        # 90% significance for n=60
        stat_val[:,1] = (np.nan, np.nan, np.nan, np.nan,
               np.nan, 1.85, 1.90, 1.92, 1.98, 2.07, 2.14,
               2.21, 2.35, 2.53, 2.63, 2.94, 3.19, 3.52, 3.85, 4.29,
               5.03, 5.74, 7.19, 9.11, 11.8, 14.4, 14.9)
        # 95% significance for n=60
        stat_val[:,2] = (np.nan, np.nan, np.nan, np.nan,
               np.nan, 2.22, 2.28, 2.33, 2.37, 2.53, 2.63,
               2.70, 2.89, 3.10, 3.28, 3.61, 3.95, 4.36, 5.04, 5.49,
               6.68, 8.20, 10.7, 13.6, 17.3, 21.2, 20.9)
        # 98% significance for n=60
        stat_val[:,3] = (np.nan, np.nan, np.nan, np.nan,
               np.nan, 2.68, 2.77, 2.85, 2.89, 3.03, 3.24,
               3.39, 3.55, 3.97, 4.05, 4.60, 5.14, 5.81, 6.91, 7.53, 
               10.1, 12.4, 18.1, 21.3, 28.4, 31.0, 29.7)
        # 99% significance for n=60
        stat_val[:,4] = (np.nan, np.nan, np.nan, np.nan,
               np.nan, 3.02, 3.20, 3.24, 3.20, 3.43, 3.66,
               3.93, 4.04, 4.57, 4.63, 5.44, 6.20, 7.13, 8.75, 9.83,
               14.4, 17.8, 25.0, 32.0, 38.4, 39.1, 37.6)

    #----------------------------------------------------------------------
    elif nobs > 67 and nobs <= 82 :
    #----------------------------------------------------------------------

        # 80% significance for n=75
        stat_val[:,0] = (np.nan, np.nan, np.nan, np.nan,
               np.nan, 1.42, 1.44, 1.46, 1.47, 1.55, 1.60,
               1.66, 1.75, 1.91, 2.03, 2.13, 2.25, 2.47, 2.65, 3.03, 
               3.26, 3.74, 4.40, 5.51, 7.28, 9.45, 10.7)
        # 90% significance for n=75
        stat_val[:,1] = (np.nan, np.nan, np.nan, np.nan,
               np.nan, 1.83, 1.88, 1.89, 1.89, 2.01, 2.09, 
               2.17, 2.28, 2.51, 2.65, 2.81, 2.97, 3.26, 3.52, 4.10, 
               4.49, 5.30, 6.43, 8.28, 11.4, 14.9, 16.4)
        # 95% significance for n=75
        stat_val[:,2] = (np.nan, np.nan, np.nan, np.nan,
               np.nan, 2.23, 2.28, 2.28, 2.31, 2.42, 2.52,
               2.64, 2.78, 3.05, 3.21, 3.45, 3.75, 3.97, 4.41, 5.31, 
               5.81, 7.07, 9.12, 12.1, 16.9, 22.0, 23.4)
        # 98% significance for n=75
        stat_val[:,3] = (np.nan, np.nan, np.nan, np.nan,
               np.nan, 2.68, 2.69, 2.76, 2.82, 2.88, 3.08,
               3.19, 3.43, 3.72, 3.99, 4.37, 4.73, 5.18, 5.65, 6.78, 
               7.76, 9.77, 14.2, 20.0, 30.3, 33.7, 32.2)
        # 99% significance for n=75
        stat_val[:,4] = (np.nan, np.nan, np.nan, np.nan,
               np.nan, 2.96, 2.98, 3.05, 3.20, 3.31, 3.44,
               3.60, 3.85, 4.29, 4.50, 4.96, 5.49, 6.00, 6.60, 8.50,
               9.76, 12.8, 20.6, 28.5, 41.2, 42.9, 39.5)


    #----------------------------------------------------------------------
    elif nobs > 82 and nobs <= 105 :
    #----------------------------------------------------------------------

        # 80% significance for n=90
        stat_val[:,0] = (np.nan, np.nan, np.nan, np.nan,
               np.nan, 1.39, 1.39, 1.43, 1.50, 1.50, 1.58,
               1.64, 1.77, 1.86, 1.96, 2.11, 2.23, 2.46, 2.61, 2.83,
               3.16, 3.59, 4.14, 5.18, 6.76, 9.01, 11.3)
        # 90% significance for n=90
        stat_val[:,1] = (np.nan, np.nan, np.nan, np.nan,
               np.nan, 1.81, 1.81, 1.84, 1.94, 1.96, 2.04,
               2.14, 2.29, 2.42, 2.56, 2.76, 2.94, 3.19, 3.49, 3.76,
               4.24, 4.85, 5.80, 7.64, 10.3, 14.9, 17.3)
        # 95% significance for n=90
        stat_val[:,2] = (np.nan, np.nan, np.nan, np.nan,
               np.nan, 2.19, 2.19, 2.31, 2.31, 2.34, 2.46,
               2.60, 2.76, 2.91, 3.11, 3.34, 3.51, 3.90, 4.28, 4.69, 
               5.36, 6.25, 7.77, 11.0, 15.4, 21.2, 24.3)
        # 98% significance for n=90
        stat_val[:,3] = (np.nan, np.nan, np.nan, np.nan,
               np.nan, 2.63, 2.61, 2.69, 2.76, 2.78, 3.01,
               3.06, 3.27, 3.57, 3.82, 4.07, 4.33, 4.79, 5.34, 5.94,
               7.05, 8.35, 11.0, 16.6, 24.6, 33.1, 33.4)
        # 99% significance for n=90
        stat_val[:,4] = (np.nan, np.nan, np.nan, np.nan,
               np.nan, 2.97, 2.94, 2.99, 3.13, 3.16, 3.37, 
               3.47, 3.70, 3.93, 4.26, 4.58, 4.92, 5.49, 6.25, 7.01, 
               8.43, 10.7, 14.1, 26.8, 35.1, 42.9, 41.4)

    #----------------------------------------------------------------------
    elif nobs > 105 and nobs <= 150 :
    #----------------------------------------------------------------------

        # 80% significance for n=120
        stat_val[:,0] = (np.nan, np.nan, np.nan, np.nan,
               np.nan, 1.38, 1.39, 1.40, 1.46, 1.48, 1.57,   
               1.63, 1.69, 1.86, 1.88, 2.07, 2.19, 2.39, 2.56, 2.75, 
               3.02, 3.36, 3.92, 4.67, 6.09, 8.59, 13.2)
        # 90% significance for n=120
        stat_val[:,1] = (np.nan, np.nan, np.nan, np.nan,
               np.nan, 1.78, 1.80, 1.83, 1.86, 1.91, 2.02,
               2.12, 2.21, 2.37, 2.47, 2.68, 2.84, 3.11, 3.33, 3.61,
               4.06, 4.53, 5.23, 6.55, 9.02, 13.4, 20.3)
        # 95% significance for n=120
        stat_val[:,2] = (np.nan, np.nan, np.nan, np.nan,
               np.nan, 2.10, 2.12, 2.19, 2.25, 2.32, 2.43, 
               2.59, 2.69, 2.86, 2.92, 3.28, 3.48, 3.78, 4.09, 4.47,
               5.04, 5.71, 6.58, 8.60, 12.9, 20.0, 27.4)
        # 98% significance for n=120
        stat_val[:,3] = (np.nan, np.nan, np.nan, np.nan,
               np.nan, 2.51, 2.51, 2.61, 2.69, 2.71, 2.92,
               3.09, 3.21, 3.42, 3.51, 3.85, 4.28, 4.58, 5.05, 5.63,
               6.24, 7.32, 8.56, 12.6, 19.1, 31.1, 37.4)
        # 99% significance for n=120
        stat_val[:,4] = (np.nan, np.nan, np.nan, np.nan,
               np.nan, 2.82, 2.83, 2.93, 3.00, 2.99, 3.30,
               3.55, 3.65, 3.95, 3.89, 4.36, 4.83, 5.17, 5.82, 6.43,
               7.29, 8.69, 10.6, 15.7, 26.5, 41.1, 46.3)

    #----------------------------------------------------------------------
    elif nobs > 150 and nobs <= 210 :
    #----------------------------------------------------------------------

        # 80% significance for n=180
        stat_val[:,0] = (np.nan, np.nan, np.nan, np.nan,
               np.nan, np.nan, 1.36, 1.38, 1.43, 1.46, 1.53,
               1.59, 1.69, 1.79, 1.92, 2.00, 2.14, 2.31, 2.50, 2.65, 
               2.91, 3.27, 3.69, 4.28, 5.41, 7.55, 13.3)
        # 90% significance for n=180
        stat_val[:,1] = (np.nan, np.nan, np.nan, np.nan,
               np.nan, np.nan, 1.73, 1.75, 1.84, 1.89, 1.96,
               2.05, 2.19, 2.33, 2.44, 2.56, 2.79, 3.02, 3.25, 3.44,
               3.79, 4.34, 4.89, 5.79, 7.40, 10.9, 20.7)
        # 95% significance for n=180
        stat_val[:,2] = (np.nan, np.nan, np.nan, np.nan,
               np.nan, np.nan, 2.08, 2.11, 2.18, 2.21, 2.38,
               2.45, 2.66, 2.76, 2.93, 3.13, 3.36, 3.70, 4.00, 4.18,
               4.67, 5.25, 5.98, 7.22, 9.50, 15.1, 28.8)
        # 98% significance for n=180
        stat_val[:,3] = (np.nan, np.nan, np.nan, np.nan,
               np.nan, np.nan, 2.52, 2.50, 2.61, 2.66, 2.83,
               2.94, 3.19, 3.31, 3.49, 3.84, 4.07, 4.39, 4.85, 4.94,
               5.62, 6.51, 7.64, 9.15, 12.9, 23.8, 42.3)
        # 99% significance for n=180
        stat_val[:,4] = (np.nan, np.nan, np.nan, np.nan,
               np.nan, np.nan, 2.80, 2.82, 2.92, 2.95, 3.16,
               3.29, 3.50, 3.76, 3.90, 4.25, 4.61, 4.90, 5.56, 5.63,
               6.43, 7.72, 8.97, 10.7, 15.8, 35.8, 54.3)

    #----------------------------------------------------------------------
    elif nobs > 210 :
    #----------------------------------------------------------------------

        # 80% significance for n=240
        stat_val[:,0] = (np.nan, np.nan, np.nan, np.nan,
               np.nan, np.nan, 1.33, 1.35, 1.42, 1.40, 1.51,
               1.57, 1.70, 1.77, 1.90, 2.02, 2.13, 2.30, 2.43, 2.72,
               2.94, 3.33, 3.55, 4.25, 5.05, 6.97, 12.5)
        # 90% significance for n=240
        stat_val[:,1] = (np.nan, np.nan, np.nan, np.nan,
               np.nan, np.nan, 1.72, 1.74, 1.84, 1.85, 1.96,
               2.03, 2.20, 2.26, 2.43, 2.56, 2.76, 2.95, 3.13, 3.54,
               3.78, 4.21, 4.67, 5.50, 6.72, 9.58, 19.8)
        # 95% significance for n=240
        stat_val[:,2] = (np.nan, np.nan, np.nan, np.nan,
               np.nan, np.nan, 2.05, 2.20, 2.20, 2.21, 2.35,
               2.43, 2.66, 2.70, 3.00, 3.11, 3.29, 3.54, 3.85, 4.25,
               4.57, 5.04, 5.62, 6.64, 8.37, 12.6, 29.5)
        # 98% significance for n=240
        stat_val[:,3] = (np.nan, np.nan, np.nan, np.nan,
               np.nan, np.nan, 2.45, 2.41, 2.59, 2.64, 2.86,
               2.96, 3.23, 3.21, 3.56, 3.79, 3.92, 4.35, 4.57, 5.10,
               5.37, 6.12, 6.98, 8.18, 11.1, 17.8, 45.8)
        # 99% significance for n=240
        stat_val[:,4] = (np.nan, np.nan, np.nan, np.nan,
               np.nan, np.nan, 2.79, 2.67, 2.88, 2.93, 3.13,
               3.28, 3.49, 3.59, 3.93, 4.22, 4.41, 4.75, 5.07, 5.62,
               5.88, 7.09, 7.96, 9.92, 13.2, 21.4, 61.5)

    return stat_val





#%%
# evaluate tcrit for modified t-test for small samples based on
# Zwiers and von Storch, 1995, J. Clim 

def lookupTcrit(r1,siglev,nobs,y=None):
    a = np.linspace(-0.35, 0.95,num=27)
    b = a - 0.025
    c = a + 0.025

    alpha = np.empty((27,3))
    alpha[:,0] = a
    alpha[:,1] = b
    alpha[:,2] = c

    stat_val = lookupTabs(nobs)
    print('siglevel')
    
    if y is None:
        if round(siglev,3) == 0.10 : 
            tab = stat_val[:,0]
        elif round(siglev,3) == 0.05 : 
            tab = stat_val[:,1]  
        elif round(siglev,3) == 0.025 : 
            tab = stat_val[:,2]  
        elif round(siglev,3) == 0.01 : 
            tab = stat_val[:,3]  
        elif round(siglev,3) == 0.005 : 
            tab = stat_val[:,4]   
        else:
            sys.exit('significance level not found in lookup table, exiting')
            
    else:
        if round(siglev,3) == 0.20 : 
            tab = stat_val[:,0]
        elif round(siglev,3) == 0.10 : 
            tab = stat_val[:,1]  
        elif round(siglev,3) == 0.05 : 
            tab = stat_val[:,2]  
        elif round(siglev,3) == 0.02 : 
            tab = stat_val[:,3]  
        elif round(siglev,3) == 0.01 : 
            tab = stat_val[:,4] 
        
            
    dims = np.shape(r1)
    nlat = dims[1]
    mlon = dims[0]
    
    tcrit = np.empty((mlon,nlat,1))
    
    for lat in range(0,nlat):
        for lon in range(0,mlon):
            r = r1[lon,lat,0]
            if r < np.min(alpha[:,1]) or r > np.max(alpha[:,2]):
                tcrit[lon,lat,0] = np.nan
                print 'Error in fct tcrit_for_modTtest_small_3d, no tcrit found'
            elif np.nanmin(np.argwhere(alpha[:,2] >= r)) and np.nanmax(np.argwhere(alpha[:,1] < 2)):
                #np.argwhere(np.asarray(c) >=5)
                tindx = np.nanmin(np.argwhere(alpha[:,2] >= r))
                tcrit[lon,lat,0] = tab[tindx]
            else:
                tcrit[lon,lat,0] = np.nan
                print 'Error in fct tcrit_for_modTtest_small_3d, no tcrit found'
    
    return tcrit

#%% 
    
# Calculate pooled sample Variance
# for modified t-test for large samples (ne>=30)
# Zwiers and von Storch, 1995, J.Clim, Eq. 13 (two sample case)

def calc_poolSampVar(x,xAve,y,yAve):
    
    dimsX = x.shape
    xnyrs = dimsX[2]
    
    ## Check if second data set has same #lat
    if y is not None:
        dimsY = y.shape
        ynyrs = dimsY[2]
    
        if dimsX[1] != dimsY[1]:
            sys.exit('Error: X and Y have different number of latitudes')
        elif dimsX[0] != dimsY[0]:
            sys.exit('Error: X and Y have different number of longitudes')
        else:
            nlat = dimsX[1]
            nlon = dimsX[0]

    xsumsq = np.zeros((nlon,nlat,1))
    ysumsq = np.zeros((nlon,nlat,1))    

    for yr in range(0,xnyrs):
        xsumsq = xsumsq+((x[:,:,[yr]] - xAve)**2) # [yr] allows to keep dimension
        
    for yr in range(0,ynyrs):
        ysumsq = ysumsq+((y[:,:,[yr]] - yAve)**2) 
          
#    sp2 = ((xnyrs-1)/xnyrs*xsumsq+(ynyrs-1)/ynyrs*ysumsq)/(xnyrs+ynyrs-2)
    sp2 = (xsumsq + ysumsq) / (xnyrs+ynyrs-2)
    
    return sp2
    
    
    
    
#%%     
# Calculate t statistic
# for modified t-test for small samples (ne<=30)
# Zwiers and von Storch, 1995, J. Clim, Eq. 16

def calc_Tstat_small(x,xAve,y,yAve):
    dimsX = x.shape
    xnyrs = dimsX[2]
    
    ## Check if second data set has same #lat
    if y is not None:
        dimsY = y.shape
        ynyrs = dimsY[2]
    
        if dimsX[1] != dimsY[1]:
            sys.exit('Error: X and Y have different number of latitudes')
        elif dimsX[0] != dimsY[0]:
            sys.exit('Error: X and Y have different number of longitudes')

        
    #Pooled sample variance, Eq. 13
    sp2 = calc_poolSampVar(x,y,xAve,yAve)

    #t-statistic, Eq. 16
    tval = (yAve-xAve)/(np.sqrt(sp2)*(np.sqrt(1/xnyrs+1/ynyrs)))


    return tval 




#%% convert variable name to string 
## usage: get_var_name(x=x) # var on both sides of =
    
def get_var_name(**kwargs): return kwargs.keys()[0]


            
##%%
#'''
#==========================================================
#CALCULATION
#==========================================================
#
#Modified t-test after Zwiers and von Storch 1995, J.Clim 8, p.336-351  
#
#'''
#import pandas as pd
#
#var = 'tas' 
#for model in '', '_e5', '_e10':
#    print 'read in exp = '+ model
##    ds_new = xr.open_dataset('/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/tas_'+ model+'.nc',decode_times=False)
#    
#    ds_new = xr.open_dataset('/Users/lenaboysen/palme/LUMIP/outdata_land_deforest-glob'+ model+'/tas_deforest-glob'+model+'.nc',decode_times=False)
#    
#    ds_new[var].shape # dsA.cveg.coords, dsA.cveg.dims, dsA.info
#    ### Data must be in shape  lon, lat, time!!
#    ds_new = ds_new.transpose('lon','lat','time')
#    ds_new[var].shape
#    
#    ## .data converts to numpy            
#    ds_new30 = ds_new[var]
#    nt = np.shape(ds_new30['time'])[0]
#    ds_new30['time'] = pd.date_range('1850-01-01', periods=nt, freq='m')
#    
#    
#    ## build annual mean 
#    ds_mean = ds_new30.groupby('time.year').mean('time')
#    
#    ## cut years of interest
#    ds_sel = ds_mean[:,:,51:82].data
#
#    if model == '':
#        ds_y = ds_sel
#    if model == '_e5' or model == '_e10':
#        ds_y = np.dstack((ds_y,ds_sel))
#
#
##ds_base = xr.open_dataset('/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/tas_ctl.nc',decode_times=False)
#ds_base = xr.open_dataset('/Users/lenaboysen/palme/LUMIP/outdata_land_control/tas_gh0001.nc',decode_times=False)
#
#ds_base[var].shape
#ds_base = ds_base.transpose('lon','lat','time')
#ds_base[var].shape
#
#ds_new = ds_base[var]
#nt2 = np.shape(ds_new['time'])[0]
#start = 1850- (nt2 - nt)/12
#ds_new['time'] = pd.date_range(str(start)+'-01-01', periods=nt2, freq='m')
### annual mean
#ds_mean = ds_new.groupby('time.year').mean('time')
#dims = ds_mean.shape
### cut years of interest
#ds_x = ds_mean[:,:,range((dims[2]-90),dims[2])].data  
#
#
#
#### modified t-Test (t-statistic = t^)
#test3d = modTtest3d(ds_x,ds_y,alternative="greater",conf_level=0.95) 
#
#'''
#t-statistic = tstat, t^ 
#df = degrees of freedom: df = n1 + n2 - 2
#pvalue : probability function for t^ and df is cut-off at the given conf_level
#    CDF(abs(tstat),df)
#    e.g. for conf_level= 0.95: if prob>=0.95 true, then stat. sign. is given
#CINT[0,:,:] contains the confidence interval: 
#    the difference between t^ and the inverse of the CDF = ppf(conf_level, df)
#    stat. significance is given if percentile of distribution is crossed
#    e.g. if cint[0,:,:] > 0 then stat. sign. is given for option "greater"
#
#    --> pvalue > conf_level == cint[0,:,:] > 0
#    
#Usage: 
#Create a mask containing 1 for stat. sign. and 0 for non-stat. sign. 
#Mask out the variable under investigation (e.g. tas)
#'''
#
#
##%% PLOT
#import cartopy.crs as ccrs
#import matplotlib.pyplot as plt
#
#def whitelinefix(lons):
#    #PlotData = np.c_[PlotData,PlotData[:,0]]
#    return np.append(lons,360)
#
#lon = ds_new['lon'].data
#lat = ds_new['lat'].data
#
#lons = whitelinefix(lon)
#lons, lats = np.meshgrid(lon, lat)
#
#
#
#
#
### Show that pval>0.95 == cint[0,:,:]>0
#cint = np.transpose(test3d['conf_interval'][0,:,:])
#pval = np.transpose(test3d['pvalue'])
#modt = cint
#modt1 = cint
#
#modt1[np.where( pval >= 0.95 )] = 1
#modt1[np.where( pval < 0 )] = 0
#
#modt[np.where( cint >0 )] = 1
#modt[np.where( cint <= 0 )] = 0
#
#diff = modt - modt1
#
#ax = plt.axes(projection=ccrs.Robinson(central_longitude=180))
#ax.coastlines()
#mesh = ax.pcolormesh(lons, lats, diff , transform = ccrs.PlateCarree(), cmap='rainbow')
#plt.colorbar(mesh,orientation = 'horizontal' ) 
#plt.show()
### --> Should be zero everywhere!
#
#
#
#
#
#### use this mask to plot Tas
#mask = (modt<=0) ## containes True / False
#tas_diff = np.transpose(ds_y.mean(axis=2) - ds_x.mean(axis=2))
#tas_masked = np.ma.masked_array(tas_diff,mask)
#
#ax = plt.axes(projection=ccrs.Robinson(central_longitude=180))
#ax.coastlines()
#mesh = ax.pcolormesh(lons, lats, tas_masked , transform = ccrs.PlateCarree(), cmap='rainbow')
#plt.colorbar(mesh,orientation = 'horizontal' ) 
#
#plt.show()
#
#
#
#
#
